@@ -11,10 +11,10 @@
 """Classes to handle Sonos UPnP Events and Subscriptions.
 
 The `Subscription` class from this module will be used in
-:py:mod:`soco.services` unless config.EVENTS_MODULE is set
-to point to :py:mod:`soco.events_twisted`, in which case
-:py:mod:`soco.events_twisted.Subscription` will be used.
-See the Example in :py:mod:`soco.events_twisted`.
+:py:mod:`soco.services` unless `config.EVENTS_MODULE` is set to
+point to :py:mod:`soco.events_twisted`, in which case
+:py:mod:`soco.events_twisted.Subscription` will be used.  See the
+Example in :py:mod:`soco.events_twisted`.
 
 Example:
 
@@ -55,6 +55,7 @@ Example:
                 sub2.unsubscribe()
                 event_listener.stop()
                 break
+
 """
 
 from __future__ import unicode_literals
@@ -65,17 +66,17 @@ import threading
 
 import requests
 
-from .compat import (
-    BaseHTTPRequestHandler, URLError, socketserver, urlopen
-)
+from .compat import BaseHTTPRequestHandler, URLError, socketserver, urlopen
 
 # Event is imported so that 'from events import Events' still works
 # pylint: disable=unused-import
 from .events_base import Event  # noqa: F401
 
 from .events_base import (
-    EventNotifyHandlerBase, EventListenerBase, SubscriptionBase,
-    SubscriptionsMap
+    EventNotifyHandlerBase,
+    EventListenerBase,
+    SubscriptionBase,
+    SubscriptionsMap,
 )
 
 from .exceptions import SoCoException
@@ -85,6 +86,7 @@ log = logging.getLogger(__name__)  # pylint: disable=C0103
 
 class EventServer(socketserver.ThreadingMixIn, socketserver.TCPServer):
     """A TCP server which handles each new request in a new thread."""
+
     allow_reuse_address = True
 
 
@@ -92,20 +94,21 @@ class EventNotifyHandler(BaseHTTPRequestHandler, EventNotifyHandlerBase):
     """Handles HTTP ``NOTIFY`` Verbs sent to the listener server.
     Inherits from `soco.events_base.EventNotifyHandlerBase`.
     """
+
     def __init__(self, *args, **kwargs):
         # The SubscriptionsMap instance created when this module is imported.
         # This is referenced by soco.events_base.EventNotifyHandlerBase.
         self.subscriptions_map = subscriptions_map
         # super appears at the end of __init__, because
         # BaseHTTPRequestHandler.__init__ does not return.
-        super(EventNotifyHandler, self).__init__(*args, **kwargs)
+        super().__init__(*args, **kwargs)
 
     def do_NOTIFY(self):  # pylint: disable=invalid-name
         """Serve a ``NOTIFY`` request by calling `handle_notification`
         with the headers and content.
         """
         headers = requests.structures.CaseInsensitiveDict(self.headers)
-        content_length = int(headers['content-length'])
+        content_length = int(headers["content-length"])
         content = self.rfile.read(content_length)
         self.handle_notification(headers, content)
         self.send_response(200)
@@ -114,8 +117,12 @@ class EventNotifyHandler(BaseHTTPRequestHandler, EventNotifyHandlerBase):
     # pylint: disable=no-self-use, missing-docstring
     def log_event(self, seq, service_id, timestamp):
         log.debug(
-            "Event %s received for %s service on thread %s at %s", seq,
-            service_id, threading.current_thread(), timestamp)
+            "Event %s received for %s service on thread %s at %s",
+            seq,
+            service_id,
+            threading.current_thread(),
+            timestamp,
+        )
 
     def log_message(self, fmt, *args):  # pylint: disable=arguments-differ
         # Divert standard webserver logging to the debug log
@@ -131,7 +138,7 @@ class EventServerThread(threading.Thread):
             address (tuple): The (ip, port) address on which the server
                 should listen.
         """
-        super(EventServerThread, self).__init__()
+        super().__init__()
         #: `threading.Event`: Used to signal that the server should stop.
         self.stop_flag = threading.Event()
         #: `tuple`: The (ip, port) address on which the server is
@@ -139,7 +146,8 @@ class EventServerThread(threading.Thread):
         self.server = server
 
     def run(self):
-        """Start the server on `address`.
+        """Start the server
+
         Handling of requests is delegated to an instance of the
         `EventNotifyHandler` class.
         """
@@ -149,8 +157,7 @@ class EventServerThread(threading.Thread):
             self.server.handle_request()
 
     def stop(self):
-        """Stop the server.
-        """
+        """Stop the server."""
         self.stop_flag.set()
 
 
@@ -163,7 +170,7 @@ class EventListener(EventListenerBase):
     """
 
     def __init__(self):
-        super(EventListener, self).__init__()
+        super().__init__()
         #: `EventServerThread`: thread on which to run.
         self._listener_thread = None
 
@@ -182,25 +189,22 @@ class EventListener(EventListenerBase):
                 should start listening.
         Returns:
             int: `requested_port_number`. Included for
-                compatibility with `soco.events_twisted.EventListener.listen`
+            compatibility with `soco.events_twisted.EventListener.listen`
 
         Note:
             The port on which the event listener listens is configurable.
             See `config.EVENT_LISTENER_PORT`
         """
-        for port_number in range(self.requested_port_number,
-                                 self.requested_port_number + 100):
+        for port_number in range(
+            self.requested_port_number, self.requested_port_number + 100
+        ):
             address = (ip_address, port_number)
             try:
                 server = EventServer(address, EventNotifyHandler)
                 break
             except OSError as oserror:
                 if oserror.errno == errno.EADDRINUSE:
-                    log.debug(
-                        "Port %s:%d is in use",
-                        ip_address,
-                        port_number
-                    )
+                    log.debug("Port %s:%d is in use", ip_address, port_number)
                 else:
                     raise
 
@@ -211,7 +215,7 @@ class EventListener(EventListenerBase):
             log.debug(
                 "The first available port %d was used instead of %d",
                 port_number,
-                self.requested_port_number
+                self.requested_port_number,
             )
         return port_number
 
@@ -221,8 +225,7 @@ class EventListener(EventListenerBase):
         self._listener_thread.stop()
         # Send a dummy request in case the http server is currently listening
         try:
-            urlopen(
-                'http://%s:%s/' % (address[0], address[1]))
+            urlopen("http://%s:%s/" % (address[0], address[1]))
         except URLError:
             # If the server is already shut down, we receive a socket error,
             # which we ignore.
@@ -231,14 +234,15 @@ class EventListener(EventListenerBase):
         # to ensure the main thread does not hang
         self._listener_thread.join(1)
         # check if join timed out and issue a warning if it did
-        if self._listener_thread.isAlive():
-            log.debug('Event Listener did not shutdown gracefully.')
+        if self._listener_thread.is_alive():
+            log.debug("Event Listener did not shutdown gracefully.")
 
 
 class Subscription(SubscriptionBase):
     """A class representing the subscription to a UPnP event.
     Inherits from `soco.events_base.SubscriptionBase`.
     """
+
     def __init__(self, service, event_queue=None):
         """
         Args:
@@ -248,7 +252,7 @@ class Subscription(SubscriptionBase):
                 events will be put. If not specified, a queue will be
                 created and used.
         """
-        super(Subscription, self).__init__(service, event_queue)
+        super().__init__(service, event_queue)
         # Used to keep track of the auto_renew thread
         self._auto_renew_thread = None
         self._auto_renew_thread_flag = threading.Event()
@@ -291,7 +295,7 @@ class Subscription(SubscriptionBase):
             `Subscription`: The Subscription instance.
 
         """
-        subscribe = super(Subscription, self).subscribe
+        subscribe = super().subscribe
         return self._wrap(subscribe, strict, requested_timeout, auto_renew)
 
     def renew(self, requested_timeout=None, is_autorenew=False, strict=True):
@@ -317,7 +321,7 @@ class Subscription(SubscriptionBase):
             `Subscription`: The Subscription instance.
 
         """
-        renew = super(Subscription, self).renew
+        renew = super().renew
         return self._wrap(renew, strict, requested_timeout, is_autorenew)
 
     def unsubscribe(self, strict=True):
@@ -337,7 +341,7 @@ class Subscription(SubscriptionBase):
             `Subscription`: The Subscription instance.
 
         """
-        unsubscribe = super(Subscription, self).unsubscribe
+        unsubscribe = super().unsubscribe
         return self._wrap(unsubscribe, strict)
 
     def _auto_renew_start(self, interval):
@@ -349,7 +353,7 @@ class Subscription(SubscriptionBase):
             """
 
             def __init__(self, interval, stop_flag, sub, *args, **kwargs):
-                super(AutoRenewThread, self).__init__(*args, **kwargs)
+                super().__init__(*args, **kwargs)
                 self.interval = interval
                 self.subscription = sub
                 self.stop_flag = stop_flag
@@ -363,7 +367,8 @@ class Subscription(SubscriptionBase):
                     subscription.renew(is_autorenew=True, strict=False)
 
         auto_renew_thread = AutoRenewThread(
-            interval, self._auto_renew_thread_flag, self)
+            interval, self._auto_renew_thread_flag, self
+        )
         auto_renew_thread.start()
 
     def _auto_renew_cancel(self):
@@ -386,8 +391,7 @@ class Subscription(SubscriptionBase):
         """
         response = None
         try:
-            response = requests.request(
-                method, url, headers=headers, timeout=3)
+            response = requests.request(method, url, headers=headers, timeout=3)
         except requests.exceptions.RequestException:
             if method != "UNSUBSCRIBE":
                 raise
@@ -436,32 +440,35 @@ class Subscription(SubscriptionBase):
                 # If an Exception occurred during execution of subscribe,
                 # renew or unsubscribe, set the cancel flag to True unless
                 # the Exception was a SoCoException upon subscribe
-                cancel = action == 'renew' or not \
-                    isinstance(exc, SoCoException)
+                cancel = action == "renew" or not isinstance(exc, SoCoException)
 
                 if cancel:
                     # If the cancel flag was set to true, cancel the
                     # subscription with an explanation.
-                    msg = "An Exception occurred. Subscription to" +\
-                        " {}, sid: {} has been cancelled".format(
-                            self.service.base_url +
-                            self.service.event_subscription_url,
-                            self.sid)
+                    msg = (
+                        "An Exception occurred. Subscription to"
+                        + " {}, sid: {} has been cancelled".format(
+                            self.service.base_url + self.service.event_subscription_url,
+                            self.sid,
+                        )
+                    )
                     self._cancel_subscription(msg)
                 # If we're not being strict, log the Exception
                 if not strict:
-                    msg = "Exception received in Subscription." +\
-                        "{} for Subscription to:\n{}, sid: {}".format(
+                    msg = (
+                        "Exception received in Subscription."
+                        + "{} for Subscription to:\n{}, sid: {}".format(
                             action,
-                            self.service.base_url +
-                            self.service.event_subscription_url,
-                            self.sid)
+                            self.service.base_url + self.service.event_subscription_url,
+                            self.sid,
+                        )
+                    )
                     log.exception(msg)
                     # If we're not being strict upon a renewal
                     # (e.g. an autorenewal) call the optional
                     # self.auto_renew_fail method, if it has been set
-                    if action == 'renew' and self.auto_renew_fail is not None:
-                        if hasattr(self.auto_renew_fail, '__call__'):
+                    if action == "renew" and self.auto_renew_fail is not None:
+                        if hasattr(self.auto_renew_fail, "__call__"):
                             # pylint: disable=not-callable
                             self.auto_renew_fail(exc)
 
