@@ -32,9 +32,7 @@ Argument(name='CurrentDateFormat', vartype='string')] ...
 
 # UPnP Spec at http://upnp.org/specs/arch/UPnP-arch-DeviceArchitecture-v1.0.pdf
 
-from __future__ import (
-    absolute_import, unicode_literals
-)
+from __future__ import absolute_import, unicode_literals
 
 import logging
 from collections import namedtuple
@@ -45,9 +43,7 @@ import requests
 from .cache import Cache
 from . import events
 from . import config
-from .exceptions import (
-    SoCoException, SoCoUPnPException, UnknownSoCoException
-)
+from .exceptions import SoCoException, SoCoUPnPException, UnknownSoCoException
 from .utils import prettify
 from .xml import XML, illegal_xml_re
 
@@ -71,36 +67,38 @@ if config.EVENTS_MODULE is None:
     config.EVENTS_MODULE = events
 
 
-class Action(namedtuple('ActionBase', 'name, in_args, out_args')):
+class Action(namedtuple("ActionBase", "name, in_args, out_args")):
     """A UPnP Action and its arguments."""
+
     def __str__(self):
-        args = ', '.join(str(arg) for arg in self.in_args)
-        returns = ', '.join(str(arg) for arg in self.out_args)
-        return '{0}({1}) -> {{{2}}}'.format(self.name, args, returns)
+        args = ", ".join(str(arg) for arg in self.in_args)
+        returns = ", ".join(str(arg) for arg in self.out_args)
+        return "{0}({1}) -> {{{2}}}".format(self.name, args, returns)
 
 
-class Argument(namedtuple('ArgumentBase', 'name, vartype')):
+class Argument(namedtuple("ArgumentBase", "name, vartype")):
     """A UPnP Argument and its type."""
+
     def __str__(self):
         argument = self.name
         if self.vartype.default:
             argument = "{0}={1}".format(self.name, self.vartype.default)
-        return '{0}: {1}'.format(argument, str(self.vartype))
+        return "{0}: {1}".format(argument, str(self.vartype))
 
 
-class Vartype(namedtuple('VartypeBase', 'datatype, default, list, range')):
+class Vartype(namedtuple("VartypeBase", "datatype, default, list, range")):
     """An argument type with default value and range."""
+
     def __str__(self):
         if self.list:
-            return '[{0}]'.format(', '.join(self.list))
+            return "[{0}]".format(", ".join(self.list))
         if self.range:
-            return '[{0}..{1}]'.format(self.range[0], self.range[1])
+            return "[{0}..{1}]".format(self.range[0], self.range[1])
         return self.datatype
 
 
 # pylint: disable=too-many-instance-attributes
 class Service(object):
-
     """A class representing a UPnP service.
 
     This is the base class for all Sonos Service classes. This class has a
@@ -108,24 +106,26 @@ class Service(object):
     defined here are dispatched automatically to the service action with the
     same name.
     """
+
     # pylint: disable=bad-continuation
     soap_body_template = (
         '<?xml version="1.0"?>'
         '<s:Envelope xmlns:s="http://schemas.xmlsoap.org/soap/envelope/"'
         ' s:encodingStyle="http://schemas.xmlsoap.org/soap/encoding/">'
-        '<s:Body>'
+        "<s:Body>"
         '<u:{action} xmlns:u="urn:schemas-upnp-org:service:'
         '{service_type}:{version}">'
-        '{arguments}'
-        '</u:{action}>'
-        '</s:Body>'
-        '</s:Envelope>')  # noqa PEP8
+        "{arguments}"
+        "</u:{action}>"
+        "</s:Body>"
+        "</s:Envelope>"
+    )  # noqa PEP8
 
     def __init__(self, soco):
         """
         Args:
             soco (SoCo): A `SoCo` instance to which the UPnP Actions will be
-                sent
+            sent
         """
 
         #: `SoCo`: The `SoCo` instance to which UPnP Actions are sent
@@ -142,14 +142,13 @@ class Service(object):
         self.version = 1
         self.service_id = self.service_type
         #: str: The base URL for sending UPnP Actions.
-        self.base_url = 'http://{}:1400'.format(self.soco.ip_address)
+        self.base_url = "http://{}:1400".format(self.soco.ip_address)
         #: str: The UPnP Control URL.
-        self.control_url = '/{}/Control'.format(self.service_type)
+        self.control_url = "/{}/Control".format(self.service_type)
         #: str: The service control protocol description URL.
-        self.scpd_url = '/xml/{}{}.xml'.format(
-            self.service_type, self.version)
+        self.scpd_url = "/xml/{}{}.xml".format(self.service_type, self.version)
         #: str: The service eventing subscription URL.
-        self.event_subscription_url = '/{}/Event'.format(self.service_type)
+        self.event_subscription_url = "/{}/Event".format(self.service_type)
         #: A cache for storing the result of network calls. By default, this is
         #: a `TimedCache` with a default timeout=0.
         self.cache = Cache(default_timeout=0)
@@ -169,25 +168,25 @@ class Service(object):
 
         # pylint: disable=invalid-name
         self.UPNP_ERRORS = {
-            400: 'Bad Request',
-            401: 'Invalid Action',
-            402: 'Invalid Args',
-            404: 'Invalid Var',
-            412: 'Precondition Failed',
-            501: 'Action Failed',
-            600: 'Argument Value Invalid',
-            601: 'Argument Value Out of Range',
-            602: 'Optional Action Not Implemented',
-            603: 'Out Of Memory',
-            604: 'Human Intervention Required',
-            605: 'String Argument Too Long',
-            606: 'Action Not Authorized',
-            607: 'Signature Failure',
-            608: 'Signature Missing',
-            609: 'Not Encrypted',
-            610: 'Invalid Sequence',
-            611: 'Invalid Control URL',
-            612: 'No Such Session',
+            400: "Bad Request",
+            401: "Invalid Action",
+            402: "Invalid Args",
+            404: "Invalid Var",
+            412: "Precondition Failed",
+            501: "Action Failed",
+            600: "Argument Value Invalid",
+            601: "Argument Value Out of Range",
+            602: "Optional Action Not Implemented",
+            603: "Out Of Memory",
+            604: "Human Intervention Required",
+            605: "String Argument Too Long",
+            606: "Action Not Authorized",
+            607: "Signature Failure",
+            608: "Signature Missing",
+            609: "Not Encrypted",
+            610: "Invalid Sequence",
+            611: "Invalid Control URL",
+            612: "No Such Session",
         }
         self.DEFAULT_ARGS = {}
 
@@ -244,14 +243,15 @@ class Service(object):
             >>> s = Service(device)
             >>> print(s.wrap_arguments([('InstanceID', 0), ('Speed', 1)]))
             <InstanceID>0</InstanceID><Speed>1</Speed>'
-            """
+        """
         if args is None:
             args = []
 
         tags = []
         for name, value in args:
             tag = "<{name}>{value}</{name}>".format(
-                name=name, value=escape("%s" % value, {'"': "&quot;"}))
+                name=name, value=escape("%s" % value, {'"': "&quot;"})
+            )
             # % converts to unicode because we are using unicode literals.
             # Avoids use of 'unicode' function which does not exist in python 3
             tags.append(tag)
@@ -294,22 +294,24 @@ class Service(object):
 
         # Get all tags in order. Elementree (in python 2.x) seems to prefer to
         # be fed bytes, rather than unicode
-        xml_response = xml_response.encode('utf-8')
+        xml_response = xml_response.encode("utf-8")
         try:
             tree = XML.fromstring(xml_response)
         except XML.ParseError:
             # Try to filter illegal xml chars (as unicode), in case that is
             # the reason for the parse error
-            filtered = illegal_xml_re.sub('', xml_response.decode('utf-8'))\
-                                     .encode('utf-8')
+            filtered = illegal_xml_re.sub("", xml_response.decode("utf-8")).encode(
+                "utf-8"
+            )
             tree = XML.fromstring(filtered)
 
         # Get the first child of the <Body> tag which will be
         # <{actionNameResponse}> (depends on what actionName is). Turn the
         # children of this into a {tagname, content} dict. XML unescaping
         # is carried out for us by elementree.
-        action_response = tree.find(
-            "{http://schemas.xmlsoap.org/soap/envelope/}Body")[0]
+        action_response = tree.find("{http://schemas.xmlsoap.org/soap/envelope/}Body")[
+            0
+        ]
         return dict((i.tag, i.text or "") for i in action_response)
 
     def compose_args(self, action_name, in_argdict):
@@ -318,16 +320,16 @@ class Service(object):
 
         Args:
             action_name (str): The name of the action to be performed.
-            in_argdict (dict): Arguments as a dict, eg
-                ``{'InstanceID': 0, 'Speed': 1}. The values
+            in_argdict (dict): Arguments as a dict, e.g.
+                ``{'InstanceID': 0, 'Speed': 1}``. The values
                 can be a string or something with a string representation.
 
         Returns:
             list: a list of ``(name, value)`` tuples.
 
         Raises:
-            `AttributeError`: If this service does not support the action.
-            `ValueError`: If the argument lists do not match the action
+            AttributeError: If this service does not support the action.
+            ValueError: If the argument lists do not match the action
                 signature.
         """
 
@@ -336,17 +338,17 @@ class Service(object):
                 # The found 'action' will be visible from outside the loop
                 break
         else:
-            raise AttributeError('Unknown Action: {0}'.format(action_name))
+            raise AttributeError("Unknown Action: {0}".format(action_name))
 
         # Check for given argument names which do not occur in the expected
         # argument list
         # pylint: disable=undefined-loop-variable
-        unexpected = set(in_argdict) - \
-            set(argument.name for argument in action.in_args)
+        unexpected = set(in_argdict) - set(argument.name for argument in action.in_args)
         if unexpected:
             raise ValueError(
-                "Unexpected argument '{0}'. Method signature: {1}"
-                .format(next(iter(unexpected)), str(action))
+                "Unexpected argument '{0}'. Method signature: {1}".format(
+                    next(iter(unexpected)), str(action)
+                )
             )
 
         # List the (name, value) tuples for each argument in the argument list
@@ -362,8 +364,9 @@ class Service(object):
             if argument.vartype.default is not None:
                 composed.append((name, argument.vartype.default))
             raise ValueError(
-                "Missing argument '{0}'. Method signature: {1}"
-                .format(argument.name, str(action))
+                "Missing argument '{0}'. Method signature: {1}".format(
+                    argument.name, str(action)
+                )
             )
         return composed
 
@@ -378,9 +381,9 @@ class Service(object):
 
         Returns:
             tuple: a tuple containing the POST headers (as a dict) and a
-                string containing the relevant SOAP body. Does not set
-                content-length, or host headers, which are completed upon
-                sending.
+            string containing the relevant SOAP body. Does not set
+            content-length, or host headers, which are completed upon
+            sending.
         """
 
         # A complete request should look something like this:
@@ -406,22 +409,27 @@ class Service(object):
 
         arguments = self.wrap_arguments(args)
         body = self.soap_body_template.format(
-            arguments=arguments, action=action, service_type=self.service_type,
-            version=self.version)
-        soap_action_template = \
+            arguments=arguments,
+            action=action,
+            service_type=self.service_type,
+            version=self.version,
+        )
+        soap_action_template = (
             "urn:schemas-upnp-org:service:{service_type}:{version}#{action}"
+        )
         soap_action = soap_action_template.format(
-            service_type=self.service_type, version=self.version,
-            action=action)
-        headers = {'Content-Type': 'text/xml; charset="utf-8"',
-                   'SOAPACTION': soap_action}
+            service_type=self.service_type, version=self.version, action=action
+        )
+        headers = {
+            "Content-Type": 'text/xml; charset="utf-8"',
+            "SOAPACTION": soap_action,
+        }
         # Note that although we set the charset to utf-8 here, in fact the
         # body is still unicode. It will only be converted to bytes when it
         # is set over the network
         return (headers, body)
 
-    def send_command(self, action, args=None, cache=None, cache_timeout=None,
-                     **kwargs):
+    def send_command(self, action, args=None, cache=None, cache_timeout=None, **kwargs):
         """Send a command to a Sonos device.
 
         Args:
@@ -437,21 +445,23 @@ class Service(object):
                 (for example if a UPnP event is received to indicate that
                 the state of the Sonos device has changed). If
                 ``cache_timeout`` is missing or `None`, the cache will use a
-                default value (which may be 0 - see `cache`). By default,
-                the cache identified by the service's `cache` attribute will
+                default value (which may be 0 - see
+                :attr:`~soco.services.Service.cache`). By default, the cache
+                identified by the service's
+                :attr:`~soco.services.Service.cache` attribute will
                 be used, but a different cache object may be specified in
-                the `cache` parameter.
+                the ``cache`` parameter.
             kwargs: Relevant arguments for the command.
 
         Returns:
              dict: a dict of ``{argument_name, value}`` items.
 
         Raises:
-            `AttributeError`: If this service does not support the action.
-            `ValueError`: If the argument lists do not match the action
+            AttributeError: If this service does not support the action.
+            ValueError: If the argument lists do not match the action
                 signature.
             `SoCoUPnPException`: if a SOAP error occurs.
-            `UnknownSoCoException`: if an unknonwn UPnP error occurs.
+            `UnknownSoCoException`: if an unknown UPnP error occurs.
             `requests.exceptions.HTTPError`: if an http error occurs.
 
         """
@@ -472,16 +482,15 @@ class Service(object):
             response = requests.post(
                 self.base_url + self.control_url,
                 headers=headers,
-                data=body.encode('utf-8'),
+                data=body.encode("utf-8"),
                 timeout=20,
             )
         except requests.exceptions.RequestException as ex:
-            raise SoCoException('Connection error: ' + str(ex))
+            raise SoCoException("Connection error: " + str(ex)) from ex
 
         log.debug("Received %s, %s", response.headers, response.text)
         status = response.status_code
-        log.debug(
-            "Received status %s from %s", status, self.soco.ip_address)
+        log.debug("Received status %s from %s", status, self.soco.ip_address)
         if status == 200:
             # The response is good. Get the output params, and return them.
             # NB an empty dict is a valid result. It just means that no
@@ -543,19 +552,19 @@ class Service(object):
         # errorDescription is not required, and Sonos does not seem to use it.
 
         # NB need to encode unicode strings before passing to ElementTree
-        xml_error = xml_error.encode('utf-8')
+        xml_error = xml_error.encode("utf-8")
         error = XML.fromstring(xml_error)
         log.debug("Error %s", xml_error)
-        error_code = error.findtext(
-            './/{urn:schemas-upnp-org:control-1-0}errorCode')
+        error_code = error.findtext(".//{urn:schemas-upnp-org:control-1-0}errorCode")
         if error_code is not None:
-            description = self.UPNP_ERRORS.get(int(error_code), '')
+            description = self.UPNP_ERRORS.get(int(error_code), "")
             raise SoCoUPnPException(
-                message='UPnP Error {} received: {} from {}'.format(
-                    error_code, description, self.soco.ip_address),
+                message="UPnP Error {} received: {} from {}".format(
+                    error_code, description, self.soco.ip_address
+                ),
                 error_code=error_code,
                 error_description=description,
-                error_xml=xml_error
+                error_xml=xml_error,
             )
 
         # Unknown error, so just return the entire response
@@ -563,16 +572,16 @@ class Service(object):
         raise UnknownSoCoException(xml_error)
 
     def subscribe(
-            self, requested_timeout=None, auto_renew=False, event_queue=None,
-            strict=True):
+        self, requested_timeout=None, auto_renew=False, event_queue=None, strict=True
+    ):
         """Subscribe to the service's events.
 
         Args:
             requested_timeout (int, optional): If requested_timeout is
                 provided, a subscription valid for that
                 number of seconds will be requested, but not guaranteed. Check
-                `Subscription.timeout` on return to find out what period of
-                validity is actually allocated.
+                :attr:`~soco.events.Subscription.timeout` on return to find out
+                what period of validity is actually allocated.
             auto_renew (bool): If auto_renew is `True`, the subscription will
                 automatically be renewed just before it expires, if possible.
                 Default is `False`.
@@ -585,19 +594,20 @@ class Service(object):
                 returned. Default `True`.
 
         Returns:
-            `Subscription`: an instance of :class:`~soco.events.Subscription`,
-                representing the new subscription. If config.EVENTS_MODULE has
-                been set to refer to :py:mod:`events_twisted`, a deferred will
-                be returned with the Subscription as its result and
-                deferred.subscription will be set to refer to the Subscription.
+            :class:`~soco.events.Subscription`: an instance of
+            :class:`~soco.events.Subscription`, representing the new
+            subscription. If config.EVENTS_MODULE has
+            been set to refer to :py:mod:`events_twisted`, a deferred will
+            be returned with the Subscription as its result and
+            deferred.subscription will be set to refer to the Subscription.
 
-        To unsubscribe, call the `unsubscribe` method on the returned object.
+        To unsubscribe, call the :meth:`~soco.events.Subscription.unsubscribe`
+        method on the returned object.
         """
-        subscription = config.EVENTS_MODULE.Subscription(
-            self, event_queue)
+        subscription = config.EVENTS_MODULE.Subscription(self, event_queue)
         return subscription.subscribe(
-            requested_timeout=requested_timeout, auto_renew=auto_renew,
-            strict=strict)
+            requested_timeout=requested_timeout, auto_renew=auto_renew, strict=strict
+        )
 
     def _update_cache_on_event(self, event):
         """Update the cache when an event is received.
@@ -625,29 +635,35 @@ class Service(object):
             action_name (str), in_args (list of Argument namedtuples,
             consisting of name and argtype), and out_args (ditto).
 
-        The return value looks like this::
-            [
-                Action(
-                    name='GetMute',
-                    in_args=[
-                        Argument(name='InstanceID', ...),
-                        Argument(
-                            name='Channel',
-                            vartype='string',
-                            list=['Master', 'LF', 'RF', 'SpeakerOnly'],
-                            range=None
-                        )
-                    ],
-                    out_args=[
-                        Argument(name='CurrentMute, ...)
-                    ]
-                )
-                Action(...)
-            ]
+        The return value looks like this:
 
-        Its string representation will look like this::
-            GetMute(InstanceID: ui4, Channel: [Master, LF, RF, SpeakerOnly]) \
-            -> {CurrentMute: boolean}
+        .. code-block:: python
+
+           [
+               Action(
+                   name='GetMute',
+                   in_args=[
+                       Argument(name='InstanceID', ...),
+                       Argument(
+                           name='Channel',
+                           vartype='string',
+                           list=['Master', 'LF', 'RF', 'SpeakerOnly'],
+                           range=None
+                       )
+                   ],
+                   out_args=[
+                       Argument(name='CurrentMute, ...)
+                   ]
+               )
+               Action(...)
+           ]
+
+        Its string representation will look like this:
+
+        .. code-block:: text
+
+           GetMute(InstanceID: ui4, Channel: [Master, LF, RF, SpeakerOnly])\n
+           -> {CurrentMute: boolean}
         """
         if self._actions is None:
             self._actions = list(self.iter_actions())
@@ -674,47 +690,46 @@ class Service(object):
 
         # pylint: disable=too-many-locals
         # pylint: disable=invalid-name
-        ns = '{urn:schemas-upnp-org:service-1-0}'
+        ns = "{urn:schemas-upnp-org:service-1-0}"
         # get the scpd body as bytes, and feed directly to elementtree
         # which likes to receive bytes
-        scpd_body = requests.get(
-            self.base_url + self.scpd_url, timeout=10).content
+        scpd_body = requests.get(self.base_url + self.scpd_url, timeout=10).content
         tree = XML.fromstring(scpd_body)
         # parse the state variables to get the relevant variable types
         vartypes = {}
-        srvStateTables = tree.findall('{}serviceStateTable'.format(ns))
+        srvStateTables = tree.findall("{}serviceStateTable".format(ns))
         for srvStateTable in srvStateTables:
-            statevars = srvStateTable.findall('{}stateVariable'.format(ns))
+            statevars = srvStateTable.findall("{}stateVariable".format(ns))
             for state in statevars:
-                name = state.findtext('{}name'.format(ns))
-                datatype = state.findtext('{}dataType'.format(ns))
-                default = state.findtext('{}defaultValue'.format(ns))
-                value_list_elt = state.find('{}allowedValueList'.format(ns))
+                name = state.findtext("{}name".format(ns))
+                datatype = state.findtext("{}dataType".format(ns))
+                default = state.findtext("{}defaultValue".format(ns))
+                value_list_elt = state.find("{}allowedValueList".format(ns))
                 if value_list_elt is None:
                     value_list_elt = ()
                 value_list = [item.text for item in value_list_elt] or None
-                value_range_elt = state.find('{}allowedValueRange'.format(ns))
+                value_range_elt = state.find("{}allowedValueRange".format(ns))
                 if value_range_elt is None:
                     value_range_elt = ()
                 value_range = [item.text for item in value_range_elt] or None
-                vartypes[name] = Vartype(datatype, default, value_list,
-                                         value_range)
+                vartypes[name] = Vartype(datatype, default, value_list, value_range)
         # find all the actions
-        actionLists = tree.findall('{}actionList'.format(ns))
+        actionLists = tree.findall("{}actionList".format(ns))
         for actionList in actionLists:
-            actions = actionList.findall('{}action'.format(ns))
+            actions = actionList.findall("{}action".format(ns))
             for i in actions:
-                action_name = i.findtext('{}name'.format(ns))
-                argLists = i.findall('{}argumentList'.format(ns))
+                action_name = i.findtext("{}name".format(ns))
+                argLists = i.findall("{}argumentList".format(ns))
                 for argList in argLists:
-                    args_iter = argList.findall('{}argument'.format(ns))
+                    args_iter = argList.findall("{}argument".format(ns))
                     in_args = []
                     out_args = []
                     for arg in args_iter:
-                        arg_name = arg.findtext('{}name'.format(ns))
-                        direction = arg.findtext('{}direction'.format(ns))
+                        arg_name = arg.findtext("{}name".format(ns))
+                        direction = arg.findtext("{}direction".format(ns))
                         related_variable = arg.findtext(
-                            '{}relatedStateVariable'.format(ns))
+                            "{}relatedStateVariable".format(ns)
+                        )
                         vartype = vartypes[related_variable]
                         if direction == "in":
                             in_args.append(Argument(arg_name, vartype))
@@ -741,191 +756,184 @@ class Service(object):
         """
 
         # pylint: disable=invalid-name
-        ns = '{urn:schemas-upnp-org:service-1-0}'
-        scpd_body = requests.get(
-            self.base_url + self.scpd_url, timeout=10).text
-        tree = XML.fromstring(scpd_body.encode('utf-8'))
+        ns = "{urn:schemas-upnp-org:service-1-0}"
+        scpd_body = requests.get(self.base_url + self.scpd_url, timeout=10).text
+        tree = XML.fromstring(scpd_body.encode("utf-8"))
         # parse the state variables to get the relevant variable types
-        statevars = tree.findall('{}stateVariable'.format(ns))
+        statevars = tree.findall("{}stateVariable".format(ns))
         for state in statevars:
             # We are only interested if 'sendEvents' is 'yes', i.e this
             # is an eventable variable
-            if state.attrib['sendEvents'] == "yes":
-                name = state.findtext('{}name'.format(ns))
-                vartype = state.findtext('{}dataType'.format(ns))
+            if state.attrib["sendEvents"] == "yes":
+                name = state.findtext("{}name".format(ns))
+                vartype = state.findtext("{}dataType".format(ns))
                 yield (name, vartype)
 
 
 class AlarmClock(Service):
-
     """Sonos alarm service, for setting and getting time and alarms."""
 
     def __init__(self, soco):
-        super(AlarmClock, self).__init__(soco)
+        super().__init__(soco)
         self.UPNP_ERRORS.update(
             {
-                801: 'Already an alarm for this time',
-            })
+                801: "Already an alarm for this time",
+            }
+        )
 
 
 class MusicServices(Service):
-
     """Sonos music services service, for functions related to 3rd party music
     services."""
 
 
-class DeviceProperties(Service):
+class AudioIn(Service):
+    """Sonos audio in service, for functions related to RCA audio input."""
 
+
+class DeviceProperties(Service):
     """Sonos device properties service, for functions relating to zones, LED
     state, stereo pairs etc."""
 
 
 class SystemProperties(Service):
-
     """Sonos system properties service, for functions relating to
     authentication etc."""
 
 
 class ZoneGroupTopology(Service):
-
     """Sonos zone group topology service, for functions relating to network
     topology, diagnostics and updates."""
 
 
 class GroupManagement(Service):
-
     """Sonos group management service, for services relating to groups."""
 
 
 class QPlay(Service):
-
     """Sonos Tencent QPlay service (a Chinese music service)"""
 
 
 class ContentDirectory(Service):
-
     """UPnP standard Content Directory service, for functions relating to
     browsing, searching and listing available music."""
 
     def __init__(self, soco):
-        super(ContentDirectory, self).__init__(soco)
+        super().__init__(soco)
         self.control_url = "/MediaServer/ContentDirectory/Control"
         self.event_subscription_url = "/MediaServer/ContentDirectory/Event"
         # For error codes, see table 2.7.16 in
         # http://upnp.org/specs/av/UPnP-av-ContentDirectory-v1-Service.pdf
-        self.UPNP_ERRORS.update({
-            701: 'No such object',
-            702: 'Invalid CurrentTagValue',
-            703: 'Invalid NewTagValue',
-            704: 'Required tag',
-            705: 'Read only tag',
-            706: 'Parameter Mismatch',
-            708: 'Unsupported or invalid search criteria',
-            709: 'Unsupported or invalid sort criteria',
-            710: 'No such container',
-            711: 'Restricted object',
-            712: 'Bad metadata',
-            713: 'Restricted parent object',
-            714: 'No such source resource',
-            715: 'Resource access denied',
-            716: 'Transfer busy',
-            717: 'No such file transfer',
-            718: 'No such destination resource',
-            719: 'Destination resource access denied',
-            720: 'Cannot process the request',
-        })
+        self.UPNP_ERRORS.update(
+            {
+                701: "No such object",
+                702: "Invalid CurrentTagValue",
+                703: "Invalid NewTagValue",
+                704: "Required tag",
+                705: "Read only tag",
+                706: "Parameter Mismatch",
+                708: "Unsupported or invalid search criteria",
+                709: "Unsupported or invalid sort criteria",
+                710: "No such container",
+                711: "Restricted object",
+                712: "Bad metadata",
+                713: "Restricted parent object",
+                714: "No such source resource",
+                715: "Resource access denied",
+                716: "Transfer busy",
+                717: "No such file transfer",
+                718: "No such destination resource",
+                719: "Destination resource access denied",
+                720: "Cannot process the request",
+            }
+        )
 
 
 class MS_ConnectionManager(Service):  # pylint: disable=invalid-name
-
     """UPnP standard connection manager service for the media server."""
 
     def __init__(self, soco):
-        super(MS_ConnectionManager, self).__init__(soco)
+        super().__init__(soco)
         self.service_type = "ConnectionManager"
         self.control_url = "/MediaServer/ConnectionManager/Control"
         self.event_subscription_url = "/MediaServer/ConnectionManager/Event"
 
 
 class RenderingControl(Service):
-
     """UPnP standard rendering control service, for functions relating to
     playback rendering, eg bass, treble, volume and EQ."""
 
     def __init__(self, soco):
-        super(RenderingControl, self).__init__(soco)
+        super().__init__(soco)
         self.control_url = "/MediaRenderer/RenderingControl/Control"
         self.event_subscription_url = "/MediaRenderer/RenderingControl/Event"
-        self.DEFAULT_ARGS.update({'InstanceID': 0})
+        self.DEFAULT_ARGS.update({"InstanceID": 0})
 
 
 class MR_ConnectionManager(Service):  # pylint: disable=invalid-name
-
     """UPnP standard connection manager service for the media renderer."""
 
     def __init__(self, soco):
-        super(MR_ConnectionManager, self).__init__(soco)
+        super().__init__(soco)
         self.service_type = "ConnectionManager"
         self.control_url = "/MediaRenderer/ConnectionManager/Control"
         self.event_subscription_url = "/MediaRenderer/ConnectionManager/Event"
 
 
 class AVTransport(Service):
-
     """UPnP standard AV Transport service, for functions relating to transport
     management, eg play, stop, seek, playlists etc."""
 
     def __init__(self, soco):
-        super(AVTransport, self).__init__(soco)
+        super().__init__(soco)
         self.control_url = "/MediaRenderer/AVTransport/Control"
         self.event_subscription_url = "/MediaRenderer/AVTransport/Event"
         # For error codes, see
         # http://upnp.org/specs/av/UPnP-av-AVTransport-v1-Service.pdf
-        self.UPNP_ERRORS.update({
-            701: 'Transition not available',
-            702: 'No contents',
-            703: 'Read error',
-            704: 'Format not supported for playback',
-            705: 'Transport is locked',
-            706: 'Write error',
-            707: 'Media is protected or not writeable',
-            708: 'Format not supported for recording',
-            709: 'Media is full',
-            710: 'Seek mode not supported',
-            711: 'Illegal seek target',
-            712: 'Play mode not supported',
-            713: 'Record quality not supported',
-            714: 'Illegal MIME-Type',
-            715: 'Content "BUSY"',
-            716: 'Resource Not found',
-            717: 'Play speed not supported',
-            718: 'Invalid InstanceID',
-            737: 'No DNS Server',
-            738: 'Bad Domain Name',
-            739: 'Server Error',
-        })
-        self.DEFAULT_ARGS.update({'InstanceID': 0})
+        self.UPNP_ERRORS.update(
+            {
+                701: "Transition not available",
+                702: "No contents",
+                703: "Read error",
+                704: "Format not supported for playback",
+                705: "Transport is locked",
+                706: "Write error",
+                707: "Media is protected or not writeable",
+                708: "Format not supported for recording",
+                709: "Media is full",
+                710: "Seek mode not supported",
+                711: "Illegal seek target",
+                712: "Play mode not supported",
+                713: "Record quality not supported",
+                714: "Illegal MIME-Type",
+                715: 'Content "BUSY"',
+                716: "Resource Not found",
+                717: "Play speed not supported",
+                718: "Invalid InstanceID",
+                737: "No DNS Server",
+                738: "Bad Domain Name",
+                739: "Server Error",
+            }
+        )
+        self.DEFAULT_ARGS.update({"InstanceID": 0})
 
 
 class Queue(Service):
-
     """Sonos queue service, for functions relating to queue management, saving
     queues etc."""
 
     def __init__(self, soco):
-        super(Queue, self).__init__(soco)
+        super().__init__(soco)
         self.control_url = "/MediaRenderer/Queue/Control"
         self.event_subscription_url = "/MediaRenderer/Queue/Event"
 
 
 class GroupRenderingControl(Service):
-
     """Sonos group rendering control service, for functions relating to group
     volume etc."""
 
     def __init__(self, soco):
-        super(GroupRenderingControl, self).__init__(soco)
+        super().__init__(soco)
         self.control_url = "/MediaRenderer/GroupRenderingControl/Control"
-        self.event_subscription_url = \
-            "/MediaRenderer/GroupRenderingControl/Event"
-        self.DEFAULT_ARGS.update({'InstanceID': 0})
+        self.event_subscription_url = "/MediaRenderer/GroupRenderingControl/Event"
+        self.DEFAULT_ARGS.update({"InstanceID": 0})
